/*  Connected components in the grid.  -- Works in shape	*/
/*  `cells' defined in this file.				*/
/*  Uses segment-bit scans.  Setup and solution only --		*/
/*  no output or control routines.				*/
/*  File:  grccsw.cs	Header:  grccsw.h			*/

/*  Axis convention:						*/
/*	   0     1       --- y --->              YMX-1		*/
/*	+-----+-----+-----+-----+-----+- . . . -+-----+		*/
/*    0	|     |     |     |     |     |         |     |		*/
/*	+-----+-----+-----+-----+-----+- . . . -+-----+		*/
/*    1	|     |     |     |     |     |         |     |		*/
/*  |	+-----+-----+-----+-----+-----+- . . . -+-----+		*/
/*  x	|     |     |     |     |     |         |     |		*/
/*  |	+-----+-----+-----+-----+-----+- . . . -+-----+		*/
/*  v	:     :     :     :     :     :         :     :		*/
/*	+-----+-----+-----+-----+-----+- . . . -+-----+		*/
/* XMX-1|     |     |     |     |     |         |     |		*/
/*	+-----+-----+-----+-----+-----+- . . . -+-----+		*/

/* Grid sizes */

#define XMX  128
#define YMX  128

/* Shape for parallel data */

extern shape [XMX][YMX]cells;	/* [128][128]	*/

/* Parallel vars for cell data */

extern int:cells
	cc,	/* Component number	*/
	cw,	/* Working component	*/
	 x,	/* x coord (axis 0)	*/
	 y;	/* y coord (axis 1)	*/

/* Parallel vars for barrier data */

extern bool:cells
	nb,	/* Northern barrier	*/
	eb,	/* Eastern barrier	*/
	wb,	/* Western barrier	*/
	sb;  /* Southern barrier	*/


/* Setup random test with probability of barrier = p/100. */
/* Returns total number of barriers installed [nb + wb]	  */
/* Expects 0 <= p <= 100.				  */
/* Must be called with `cells' as current shape.	  */

int setup(int p);

/* Calculate connected components.		    */
/* Returns no. of cycles, not counting final test.  */
/* Must be called with `cells' as current shape.    */

int gccs();


int gccs_growing ( int:current clr, int cc, int *cy, int:current root_clr );

int gccx_growing ( int:current cc, int cl, int *cy, int:current root_clr );

